<?php

namespace Drupal\selective_better_exposed_filters\Plugin\better_exposed_filters\filter;

use Drupal\better_exposed_filters\Plugin\better_exposed_filters\filter\RadioButtons as RadioButtonsBase;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Plugin\views\filter\TaxonomyIndexTid;
use Drupal\views\Views;

/**
 * Default widget implementation.
 *
 * @BetterExposedFiltersFilterWidget(
 *   id = "bef",
 *   label = @Translation("Checkboxes/Radio Buttons"),
 * )
 */
class RadioButtons extends RadioButtonsBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'options_show_only_used' => FALSE,
      'options_show_only_used_filtered' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\views\Plugin\views\filter\FilterPluginBase $filter */
    $filter = $this->handler;

    $form = parent::buildConfigurationForm($form, $form_state);

    if ($filter->isExposed() && $filter instanceof TaxonomyIndexTid) {
      $form['options_show_only_used'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Show only used terms'),
        '#default_value' => !empty($this->configuration['options_show_only_used']),
        '#description' => $this->t('Restrict exposed filter values to those presented in the result set.'),
      ];

      $form['options_show_only_used_filtered'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Filter terms based on filtered result set'),
        '#default_value' => !empty($this->configuration['options_show_only_used_filtered']),
        '#description' => $this->t('Restrict exposed filter values to those presented in the already filtered result set.'),
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function exposedFormAlter(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\views\Plugin\views\filter\FilterPluginBase $filter */
    $filter = $this->handler;
    parent::exposedFormAlter($form, $form_state);

    if ($filter->isExposed() && !empty($this->configuration['options_show_only_used'])) {
      $identifier = $filter->options['is_grouped'] ? $filter->options['group_info']['identifier'] : $filter->options['expose']['identifier'];

      if (empty($this->view->selective_filter)) {
        $view = Views::getView($this->view->id());
        $view->selective_filter = TRUE;
        $view->setArguments($this->view->args);
        $view->setItemsPerPage(0);
        $view->setDisplay($this->view->current_display);
        $view->preExecute();
        $view->execute();

        if (!empty($view->result)) {
          $hierarchy = !empty($filter->options['hierarchy']);
          $field_id = $filter->configuration['field_name'];
          $relationship = $filter->options['relationship'];
          $element = &$form[$identifier];

          $tids = ['All' => 'All'];
          foreach ($view->result as $row) {
            $entity = ($relationship == 'none') ? $row->_entity : ($row->_relationship_entities[$relationship] ?? FALSE);
            if ($entity instanceof FieldableEntityInterface && $entity->hasField($field_id)) {
              $term_values = $entity->get($field_id)->getValue();

              if (!empty($term_values)) {
                foreach ($term_values as $term_value) {
                  $tid = $term_value['target_id'];
                  $tids[$tid] = $tid;

                  if ($hierarchy) {
                    $parents = \Drupal::service('entity_type.manager')
                      ->getStorage("taxonomy_term")
                      ->loadAllParents($tid);

                    /** @var \Drupal\taxonomy\TermInterface $term */
                    foreach ($parents as $term) {
                      $tids[$term->id()] = $term->id();
                    }
                  }
                }
              }
            }
          }

          $element['#options'] = array_intersect_key($element['#options'], $tids);
        }
      }
      else {
        if (!empty($this->configuration['options_show_only_used_filtered'])) {
          $user_input = $form_state->getUserInput();
          if (isset($user_input[$identifier])) {
            unset($user_input[$identifier]);
          }
        }
        else {
          $user_input = [];
        }
        $form_state->setUserInput($user_input);
      }
    }
  }

}
